#!/usr/local/bin/python

"""
A python implementation for both absort sorting algorithm versions: normal (absort)
and extended (eabsort).

Alberto Bertogli - albertogli@altavista.net
9/2000 - Pinamar, Buenos Aires, Argentina
http://absort.sourceforge.net
"""

def eabsort(s, m):
	"""
The extended absort function

s is the stuff to sort, including sentinel key (see below)
m is len(s)-1
General sort, not integer sort.

Sorting [ '33', '4', '4'] will turn out into ['33', '4', '4'] 
instead of ['4', '4', '33'].
That's python behaviour, perfectly right:
33 > 4 == 1; but "33" > "4" == 0

It wont be a problem if you pass it an integer list,
the only restriction is that s elements have to be able to be
assigned in-place (s[1] = s[2]). Lists are _the_ type.
Note there is not any kind of error-checking inside the function.
Sentinel key is also your problem, take good care of it (in case you
dont know what i'm talking about, you only need to know the sentinel
key means that the first element of the list has to be the minimum
possible value, in case of strings this is chr(0) in every platform,
but integers differs with the arch. Longs are specially difficult in
python because they are unlimited).
	"""
	i = 1 
	n = 0
	while n < m:
		if s[n] > s[n+1]:
			s[n], s[n+1] = s[n+1], s[n]
			n = n - 1
			i = i + 1
		else:
			n = n + i
			i = 1
	return s


def absort(s, m):
	"""
The absort sort function

s is the stuff to sort, including sentinel key (read eabsort docs. )
m is len(s)-1

Please read the eabsort() documentation,
if you want to know what's the difference between this two versions, go to
http://absort.sourceforge.net or email albertogli@altavista.net.
You should use this function only if you have a good reason to,
else i extremely recommend you the extended version, it's much, much faster
at the cost of only one more variable, and you still can add elements while sorting.
This is mainly for demonstration only.. if you really wanted to add elements
while sorting you should, almost for sure, modify this.
	"""
	n = 0
	while n < m:
		if s[n] > s[n+1]:
			s[n], s[n+1] = s[n+1], s[n]
			n = n - 1
		else:
			n = n + 1
	return s



# main, used if called directly and not as a module
if __name__ == '__main__': 
	import sys
	s = [chr(0)] 			# Supposed to be the min. possible value, our sentinel key
	argc = len(sys.argv) 	# From C
	if argc <= 1: 
		print "No arguments!"
		sys.exit(1)
	
	for i in range(1, argc): 
		s.append(sys.argv[i])
	
	t = s
	
	s = eabsort(s, argc-1)
	print '\neabsort: %s' % s
	
	t = absort(t, argc-1)
	print '\nabsort:  %s\n' % s


